@@ -1,5 +1,6 @@
 #
 # Copyright 2014 Andrew Ayer
+# Copyright 2016-2020 Chris Lamb <lamby@debian.org>
 #
 # This file is part of strip-nondeterminism.
 #
@@ -24,6 +25,16 @@ use warnings;
 use File::Temp;
 use File::StripNondeterminism;
 use Archive::Zip qw/:CONSTANTS :ERROR_CODES/;
+use Sub::Override;
+
+=head1 DEPRECATION PLANS
+
+Given that so many libraries and utilities generate ZIP archives themselves
+(rather than delegating to one or two central libraries) it is infeasible to
+get this upstreamed everywhere. There are also countless formats that generate
+ZIP files under different names (notably Jar archives and its many variants).
+
+=cut
 
 # A magic number from Archive::Zip for the earliest timestamp that
 # can be represented by a Zip file.  From the Archive::Zip source:
@@ -76,11 +87,6 @@ sub normalize_member($$) {
 	return 1;
 }
 
-use constant {
-	CENTRAL_HEADER => 0,
-	LOCAL_HEADER => 1
-};
-
 sub unixtime_to_winnt($) {
 	my $unixtime = shift || 0;
 
@@ -91,11 +97,10 @@ sub unixtime_to_winnt($) {
 	return $unixtime + $secondsdiff;
 }
 
-sub normalize_extra_fields($$$) {
+sub normalize_extra_fields($$) {
 	# See http://sources.debian.net/src/zip/3.0-6/proginfo/extrafld.txt for extra field documentation
-	# $header_type is CENTRAL_HEADER or LOCAL_HEADER.
 	# WARNING: some fields have a different format depending on the header type
-	my ($canonical_time, $field, $header_type) = @_;
+	my ($canonical_time, $field) = @_;
 
 	my $result = "";
 	my $pos = 0;
@@ -134,12 +139,12 @@ sub normalize_extra_fields($$$) {
 			}
 		} elsif ($id == 0x7875) { # Info-ZIP New Unix Extra Field
 			$result .= substr($field, $pos, 4);
-		  #  Version       1 byte      version of this extra field, currently 1
-		  #  UIDSize       1 byte      Size of UID field
-		  #  UID           Variable    UID for this entry
-		  #  GIDSize       1 byte      Size of GID field
-		  #  GID           Variable    GID for this entry
-		  # (Same format for both central header and local header)
+			#  Version       1 byte      version of this extra field, currently 1
+			#  UIDSize       1 byte      Size of UID field
+			#  UID           Variable    UID for this entry
+			#  GIDSize       1 byte      Size of GID field
+			#  GID           Variable    GID for this entry
+			# (Same format for both central header and local header)
 			if (ord(substr($field, $pos + 4, 1)) == 1) {
 				my $uid_len = ord(substr($field, $pos + 5, 1));
 				my $gid_len = ord(substr($field, $pos + 6 + $uid_len, 1));
@@ -149,6 +154,16 @@ sub normalize_extra_fields($$$) {
 				$result .= substr($field, $pos + 4, $len);
 			}
 		} else {
+			# Catch invalid field lengths by calculating whether we would
+			# read beyond the end of the file.
+			if (!defined($len)) {
+				warn "strip-nondeterminism: unknown extra field length";
+				return;
+			}
+			if ($pos + $len >= length($field)) {
+				warn "strip-nondeterminism: invalid extra field length ($len)";
+				return;
+			}
 			# use the current extra field unmodified.
 			$result .= substr($field, $pos, $len+4);
 		}
@@ -195,8 +210,22 @@ sub normalize {
 	$canonical_time = SAFE_EPOCH
 	  if not defined $canonical_time or $canonical_time < SAFE_EPOCH;
 	my @filenames = sort $filename_cmp $zip->memberNames();
+	if (exists($options{filename_filter})) {
+		@filenames = $options{filename_filter}->(@filenames);
+		# Remove any members deleted by the custom filter
+		my %seen = map { $_ => 1 } @filenames;
+		for my $filename ($zip->memberNames()) {
+			next if $seen{$filename};
+			print STDERR "Removing $filename from $zip_filename\n";
+			$zip->removeMember($filename);
+		}
+	}
 	for my $filename (@filenames) {
 		my $member = $zip->removeMember($filename);
+		if ($member->isEncrypted()) {
+			warn "strip-nondeterminism: $zip_filename: ignoring encrypted zip file\n";
+			return 0;
+		}
 		$zip->addMember($member);
 		# member_normalizer returns the timestamp to use.
 		my $timestamp = exists $options{member_normalizer}
@@ -209,13 +238,37 @@ sub normalize {
 				? oct(755)
 				: oct(644));
 		}
-		$member->cdExtraField(
-			normalize_extra_fields($canonical_time, $member->cdExtraField(), CENTRAL_HEADER));
-		$member->localExtraField(
-			normalize_extra_fields($canonical_time, $member->localExtraField(), LOCAL_HEADER));
 	}
 	my $old_perms = (stat($zip_filename))[2] & oct(7777);
-	$zip->overwrite();
+
+	# Archive::Zip::Member does not handle the localExtraField field (used for
+	# uid/gids) correctly or consistently.
+	#
+	# It does not populate localExtraField in the class upon initial reading of
+	# the file whilst it does for cdExtraField. One can workaround this
+	# manually with calls to _seekToLocalHeader and _readLocalFileHeader but
+	# upon writing to a file back to the disk Archive::Zip will ignore any
+	# stored value of localExtraField (!) and reload it again from the existing
+	# file handle in/around rewindData.
+	#
+	# We therefore override the accessor methods of the Member class to
+	# ensure that normalised values are used in this final save.
+	#
+	# <https://salsa.debian.org/reproducible-builds/strip-nondeterminism/issues/4>
+	my @overrides = map {
+		my $full_name = "Archive::Zip::Member::$_";
+		my $orig_sub = \&$full_name;
+		Sub::Override->new(
+			$full_name => sub {
+				my $result = $orig_sub->(@_);
+				return defined($result) ?
+					normalize_extra_fields($canonical_time, $result) : $result;
+			}
+		);
+	} qw(cdExtraField localExtraField);
+
+	return 0 unless $zip->overwrite() == AZ_OK;
+	$_->restore for @overrides;
 	chmod($old_perms, $zip_filename);
 	return 1;
 }
